import java.util.concurrent.Semaphore;

public class Pieza {
	
	private int pieza;
	private int nPistas;	
	private boolean[] pistasOcupadas;
	private int nPistasLibres;
	private boolean isUltimaPieza;
	private Semaphore semaforo; // Controla si la pieza est llena de coches
	private Semaphore semaforoBuscarPista; // Controla la seccin crtica de pistasOcupadas y nPistasLibres
	
	Pieza(int numeroPieza, boolean pistasAleatorias, int numeroMaximoPistas) {
		pieza = numeroPieza;
		if (pistasAleatorias) nPistas = 1+generarAleatorio(numeroMaximoPistas-1);
		else nPistas=numeroMaximoPistas;
		nPistasLibres = nPistas;
		isUltimaPieza=false;
		pistasOcupadas = new boolean[nPistas]; // Inicializado automaticamente a false
		semaforo = new Semaphore(nPistas);
		semaforoBuscarPista = new Semaphore(1);
	}
	
	// Marcamos esta pieza como la ltima del circuito
	void setUltimaPieza() {
		isUltimaPieza=true;
	}
	
	// Genera un aleatorio entre 0 y limite
	int generarAleatorio(int limite) {
		return((int)(Math.round((Math.random()*(limite)))));
	}
	
	// Devuelve el numero de pistas de la pieza
	int getPistas() {
		return(nPistas);
	}
	
	// Busca y reserva una de las pistas libres
	// Precondicin: hay alguna pista libre
	int entrarPista(int coche) throws InterruptedException {
		semaforoBuscarPista.acquire();
		int ordinal = generarAleatorio(nPistasLibres-1); // Numero de orden de la pista libre que vamos a coger
		int i=0;
		int pista=0;
		for (i=0; i<nPistas; i++) {
			if (!pistasOcupadas[i]) {
				if (pista==ordinal) break;
				else pista++;
			}
		}
		pistasOcupadas[i]=true;
		nPistasLibres--;
		System.out.println("Coche " + coche + " entra a la pista " + i + " de la pieza " + pieza);
		semaforoBuscarPista.release();
		return(i);
	}
	
	// Desocupa una pista
	void salirPista(int pista, int coche) throws InterruptedException {
		semaforoBuscarPista.acquire();
		pistasOcupadas[pista]=false;
		nPistasLibres++;
		if (isUltimaPieza) System.out.println("Coche " + coche + " ha llegado a la meta");
		semaforoBuscarPista.release();
	}
	
	// El coche recorre la pieza. Este es el mtodo principal al que hay que llamar
	// desde fuera de la pieza
	void recorrerPieza(int coche) throws InterruptedException {
		semaforo.acquire();
		int pista = entrarPista(coche);
		Thread.sleep(1000+generarAleatorio(2000));
		salirPista(pista, coche);
		semaforo.release();
	}
	
}
